@@ -11,20 +11,19 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 
-from tests import unittest, BaseSessionTest
+from tests import mock, unittest, BaseSessionTest
 
 import base64
-import mock
 import copy
 import os
 import json
+import pytest
 
 import ibm_botocore
 import ibm_botocore.session
 from ibm_botocore.compat import OrderedDict
 from ibm_botocore.exceptions import ParamValidationError, MD5UnavailableError
 from ibm_botocore.exceptions import AliasConflictParameterError
-from ibm_botocore.exceptions import MissingServiceIdError
 from ibm_botocore.awsrequest import AWSRequest
 from ibm_botocore.compat import quote, six
 from ibm_botocore.config import Config
@@ -32,10 +31,8 @@ from ibm_botocore.docs.bcdoc.restdoc import DocumentStructure
 from ibm_botocore.docs.params import RequestParamsDocumenter
 from ibm_botocore.docs.example import RequestExampleDocumenter
 from ibm_botocore.hooks import HierarchicalEmitter
-from ibm_botocore.loaders import Loader
 from ibm_botocore.model import OperationModel, ServiceModel, ServiceId
 from ibm_botocore.model import DenormalizedStructureBuilder
-from ibm_botocore.session import Session
 from ibm_botocore.signers import RequestSigner
 from ibm_botocore.credentials import Credentials
 from ibm_botocore.utils import conditionally_calculate_md5
@@ -841,7 +838,7 @@ class TestRetryHandlerOrder(BaseSessionTest):
         operation = service_model.operation_model('CopyObject')
         responses = client.meta.events.emit(
             'needs-retry.s3.CopyObject',
-            request_dict={},
+            request_dict={'context': {}},
             response=(mock.Mock(), mock.Mock()), endpoint=mock.Mock(),
             operation=operation, attempts=1, caught_exception=None)
         # This is implementation specific, but we're trying to verify that
@@ -1053,6 +1050,32 @@ class TestAddMD5(BaseMD5Test):
             request_dict['headers']['Content-MD5'],
             'OFj2IjCsPJFfMAxmQxLGPw==')
 
+    # IBM Unsupported
+    # def test_skip_md5_when_flexible_checksum_context(self):
+    #     request_dict = {
+    #         'body': six.BytesIO(b'foobar'),
+    #         'headers': {},
+    #         'context': {
+    #             'checksum': {
+    #                 'request_algorithm': {
+    #                     'in': 'header',
+    #                     'algorithm': 'crc32',
+    #                     'name': 'x-amz-checksum-crc32',
+    #                 }
+    #             }
+    #         }
+    #     }
+    #     conditionally_calculate_md5(request_dict)
+    #     self.assertNotIn('Content-MD5', request_dict['headers'])
+
+    # def test_skip_md5_when_flexible_checksum_explicit_header(self):
+    #     request_dict = {
+    #         'body': six.BytesIO(b'foobar'),
+    #         'headers': {'x-amz-checksum-crc32': 'foo'},
+    #     }
+    #     conditionally_calculate_md5(request_dict)
+    #     self.assertNotIn('Content-MD5', request_dict['headers'])
+
 
 class TestParameterAlias(unittest.TestCase):
     def setUp(self):
@@ -1119,10 +1142,10 @@ class TestParameterAlias(unittest.TestCase):
             self.sample_section
         )
         contents = self.sample_section.flush_structure().decode('utf-8')
-        self.assertIn(':type ' + self.alias_name + ':',  contents)
-        self.assertIn(':param ' + self.alias_name + ':',  contents)
-        self.assertNotIn(':type ' + self.original_name + ':',  contents)
-        self.assertNotIn(':param ' + self.original_name + ':',  contents)
+        self.assertIn(':type ' + self.alias_name + ':', contents)
+        self.assertIn(':param ' + self.alias_name + ':', contents)
+        self.assertNotIn(':type ' + self.original_name + ':', contents)
+        self.assertNotIn(':param ' + self.original_name + ':', contents)
 
     def test_alias_parameter_in_documentation_request_example(self):
         RequestExampleDocumenter(
@@ -1133,7 +1156,7 @@ class TestParameterAlias(unittest.TestCase):
             self.sample_section
         )
         contents = self.sample_section.flush_structure().decode('utf-8')
-        self.assertIn(self.alias_name + '=',  contents)
+        self.assertIn(self.alias_name + '=', contents)
         self.assertNotIn(self.original_name + '=', contents)
 
 
@@ -1181,10 +1204,29 @@ class TestPrependToHost(unittest.TestCase):
 
     def test_does_validate_long_host(self):
         with self.assertRaises(ParamValidationError):
-           self._prepend_to_host(
-               'https://example.com/path', 'toolong'*100)
+            self._prepend_to_host('https://example.com/path', 'toolong' * 100)
 
     def test_does_validate_host_with_illegal_char(self):
         with self.assertRaises(ParamValidationError):
-           self._prepend_to_host(
-               'https://example.com/path', 'host#name')
+            self._prepend_to_host('https://example.com/path', 'host#name')
+
+
+@pytest.mark.parametrize(
+    'environ, header_before, header_after',
+    [
+        ({'AWS_LAMBDA_FUNCTION_NAME': 'foo'}, {}, {}),
+        ({'_X_AMZ_TRACE_ID': 'bar'}, {}, {}),
+        ({'AWS_LAMBDA_FUNCTION_NAME': 'foo', '_X_AMZ_TRACE_ID': 'bar'},
+         {}, {'X-Amzn-Trace-Id': 'bar'}),
+        ({'AWS_LAMBDA_FUNCTION_NAME': 'foo', '_X_AMZ_TRACE_ID': 'bar'},
+         {'X-Amzn-Trace-Id': 'fizz'}, {'X-Amzn-Trace-Id': 'fizz'}),
+        ({'AWS_LAMBDA_FUNCTION_NAME': 'foo',
+          '_X_AMZ_TRACE_ID': 'first\nsecond'},
+         {}, {'X-Amzn-Trace-Id': 'first%0Asecond'})
+    ]
+)
+def test_add_recursion_detection_header(environ, header_before, header_after):
+    request_dict = {'headers': header_before}
+    with mock.patch('os.environ', environ):
+        handlers.add_recursion_detection_header(request_dict)
+        assert request_dict['headers'] == header_after
